/// <reference types="node" />
/// <reference types="node" />
/// <reference types="node" />
/** @reflection never */
import { Socket } from 'net';
import { DatabaseError, DatabaseTransaction, SelectorState } from '@deepkit/orm';
import { ReceiveType, Type } from '@deepkit/type';
import { TLSSocket } from 'tls';
import { Host, PostgresClientConfig } from './config.js';
import { DefaultPlatform, PreparedEntity } from '@deepkit/sql';
export declare enum CommonTypes {
    bool = 16,
    bytea = 17,
    char = 18,
    name = 19,
    int8 = 20,
    int2 = 21,
    int2vector = 22,
    int4 = 23,
    regproc = 24,
    text = 25,
    oid = 26,
    tid = 27,
    xid = 28,
    cid = 29,
    oidvector = 30,
    json = 114,
    xml = 142,
    _xml = 143,
    pg_node_tree = 194,
    _json = 199,
    _xid8 = 271,
    point = 600,
    lseg = 601,
    path = 602,
    box = 603,
    polygon = 604,
    line = 628,
    _line = 629,
    cidr = 650,
    _cidr = 651,
    float4 = 700,
    float8 = 701,
    unknown = 705,
    circle = 718,
    _circle = 719,
    money = 790,
    _money = 791,
    macaddr = 829,
    inet = 869,
    _bool = 1000,
    _bytea = 1001,
    _char = 1002,
    _name = 1003,
    _int2 = 1005,
    _int2vector = 1006,
    _int4 = 1007,
    _regproc = 1008,
    _text = 1009,
    _tid = 1010,
    _xid = 1011,
    _cid = 1012,
    _oidvector = 1013,
    _bpchar = 1014,
    _varchar = 1015,
    _int8 = 1016,
    _point = 1017,
    _lseg = 1018,
    _path = 1019,
    _box = 1020,
    _float4 = 1021,
    _float8 = 1022,
    _polygon = 1027,
    _oid = 1028,
    aclitem = 1033,
    _aclitem = 1034,
    _macaddr = 1040,
    _inet = 1041,
    bpchar = 1042,
    varchar = 1043,
    date = 1082,
    time = 1083,
    timestamp = 1114,
    _timestamp = 1115,
    _date = 1182,
    _time = 1183,
    timestamptz = 1184,
    _timestamptz = 1185,
    interval = 1186,
    _interval = 1187,
    _numeric = 1231,
    _cstring = 1263,
    timetz = 1266,
    _timetz = 1270,
    bit = 1560,
    _bit = 1561,
    varbit = 1562,
    _varbit = 1563,
    numeric = 1700,
    record = 2249,
    cstring = 2275,
    any = 2276,
    anyarray = 2277,
    void = 2278,
    trigger = 2279,
    language_handler = 2280,
    internal = 2281,
    anyelement = 2283,
    _record = 2287,
    anynonarray = 2776,
    pg_authid = 2842,
    pg_auth_members = 2843,
    _txid_snapshot = 2949,
    uuid = 2950,
    _uuid = 2951,
    txid_snapshot = 2970,
    fdw_handler = 3115,
    tsm_handler = 3310,
    pg_ndistinct = 3361,
    pg_dependencies = 3402,
    anyenum = 3500,
    tsvector = 3614,
    tsquery = 3615,
    gtsvector = 3642,
    _tsvector = 3643,
    _gtsvector = 3644,
    _tsquery = 3645,
    regconfig = 3734,
    _regconfig = 3735,
    regdictionary = 3769,
    _regdictionary = 3770,
    jsonb = 3802,
    _jsonb = 3807,
    anyrange = 3831,
    event_trigger = 3838,
    int4range = 3904,
    _int4range = 3905,
    numrange = 3906,
    _numrange = 3907,
    tsrange = 3908,
    _tsrange = 3909,
    tstzrange = 3910,
    _tstzrange = 3911,
    daterange = 3912,
    _daterange = 3913,
    int8range = 3926,
    _int8range = 3927,
    pg_shseclabel = 4066,
    jsonpath = 4072,
    _jsonpath = 4073
}
export declare class PostgresError extends DatabaseError {
}
export declare class PostgresConnectionError extends PostgresError {
}
export interface Result {
    rows: any[];
    rowCount: number;
}
export declare function readMessageSize(buffer: Uint8Array | ArrayBuffer, offset?: number): number;
export declare class ResponseParser {
    protected readonly onMessage: (response: Uint8Array) => void;
    protected currentMessage?: Uint8Array;
    protected currentMessageSize: number;
    constructor(onMessage: (response: Uint8Array) => void);
    feed(data: Uint8Array, bytes?: number): void;
}
export declare enum ConnectionStatus {
    pending = "pending",
    connecting = "connecting",
    connected = "connected",
    disconnected = "disconnected"
}
export interface ConnectionRequest {
    readonly?: boolean;
    exclusive?: boolean;
    transaction?: PostgresDatabaseTransaction;
    hint?: string;
}
export declare const enum HostType {
    primary = 0,
    secondary = 1
}
export type TransactionTypes = 'REPEATABLE READ' | 'READ COMMITTED' | 'SERIALIZABLE';
export declare class PostgresDatabaseTransaction extends DatabaseTransaction {
    reservation?: ConnectionReservation;
    setTransaction?: TransactionTypes;
    /**
     * This is the default for mysql databases.
     */
    repeatableRead(): this;
    readCommitted(): this;
    serializable(): this;
    begin(): Promise<void>;
    commit(): Promise<void>;
    rollback(): Promise<void>;
}
type Deserializer = (message: Uint8Array) => any;
type DescribeColumn = {
    name: string;
    typeName: string;
    type: CommonTypes;
    columnObjectId: number;
    columnAttributeNumber: number;
    typeSize: number;
    typeModifier: number;
    format: number;
};
interface CommandCurrent {
    resolve: Function;
    reject: Function;
    deserializer: Deserializer;
    next?: CommandCurrent;
}
export declare abstract class Command<T> {
    protected head?: CommandCurrent;
    protected tail?: CommandCurrent;
    protected description: DescribeColumn[];
    write: (buffer: Uint8Array) => void;
    sendAndWait(message: Uint8Array, deserializer?: Deserializer): Promise<void>;
    next(): void;
    query<T>(sql: string, type?: ReceiveType<T>): Promise<any>;
    abstract execute(host: Host, connection: PostgresClientConnection, transaction?: PostgresDatabaseTransaction): Promise<T>;
    needsWritableHost(): boolean;
    handleDescription(description: DescribeColumn[]): void;
    handleResponse(response: Uint8Array): boolean;
}
export declare class SimpleQueryCommand extends Command<any[]> {
    protected sql: string;
    message: Uint8Array;
    constructor(sql: string);
    execute(host: Host, connection: PostgresClientConnection, transaction?: PostgresDatabaseTransaction): Promise<never[]>;
}
export declare class InsertCommand extends Command<number> {
    platform: DefaultPlatform;
    prepared: PreparedEntity;
    items: any[];
    constructor(platform: DefaultPlatform, prepared: PreparedEntity, items: any[]);
    execute(host: Host, client: PostgresClientConnection, transaction?: PostgresDatabaseTransaction): Promise<number>;
}
type DebugResult = {
    values: any[];
    rows: string[];
    types: string[];
    columns: DescribeColumn[];
};
/**
 * This command just exists for us to debug the binary responses.
 */
export declare class DebugSelectCommand extends Command<DebugResult> {
    protected sql: string;
    protected type: Type;
    deserialize?: (message: Uint8Array, rows: any[]) => any;
    constructor(sql: string, type: Type);
    handleDescription(description: DescribeColumn[]): void;
    execute(host: Host, connection: PostgresClientConnection, transaction?: PostgresDatabaseTransaction): Promise<DebugResult>;
    handleResponse(response: Uint8Array): boolean;
}
export declare class FindCommand<T> extends Command<T[]> {
    sql: string;
    selector: SelectorState;
    private statementName;
    cache?: Buffer;
    created: boolean;
    deserialize?: (message: Uint8Array, rows: any[]) => any;
    constructor(sql: string, selector: SelectorState, statementName?: string);
    setParameters(params: any[]): void;
    handleDescription(description: DescribeColumn[]): void;
    execute(host: Host, connection: PostgresClientConnection, transaction?: PostgresDatabaseTransaction): Promise<any[]>;
}
export declare class PostgresStats {
    /**
     * How many connections have been created.
     */
    connectionsCreated: number;
    /**
     * How many connections have been reused.
     */
    connectionsReused: number;
    /**
     * How many connection requests were queued because pool was full.
     */
    connectionsQueued: number;
    bytesReceived: number;
    bytesSent: number;
}
export declare class PostgresConnectionPool {
    protected config: PostgresClientConfig;
    protected stats: PostgresStats;
    protected connectionId: number;
    /**
     * Connections, might be in any state, not necessarily connected.
     */
    connections: PostgresClientConnection[];
    protected hints: {
        [key: string]: PostgresClientConnection;
    };
    protected queue: {
        resolve: (connection: PostgresClientConnection) => void;
        request: ConnectionRequest;
    }[];
    protected lastError?: Error;
    constructor(config: PostgresClientConfig, stats: PostgresStats);
    protected waitForAllConnectionsToConnect(throws?: boolean): Promise<void>;
    connect(): Promise<void>;
    close(): void;
    protected ensureHostsConnectedPromise?: Promise<void>;
    ensureHostsConnected(throws?: boolean): Promise<void>;
    protected findHostForRequest(hosts: Host[], request: ConnectionRequest): Host;
    protected createAdditionalConnectionForRequest(request: ConnectionRequest): PostgresClientConnection;
    protected newConnection(host: Host): PostgresClientConnection;
    protected matchRequest(connection: PostgresClientConnection, request: ConnectionRequest): boolean;
    isConnectionUsable(connection: PostgresClientConnection): boolean;
    /**
     * Returns an existing or new connection, that needs to be released once done using it.
     */
    protected getConnection(request?: ConnectionRequest): Promise<PostgresClientConnection>;
    protected release(connection: PostgresClientConnection): void;
    protected releaseReservation(connection: PostgresClientConnection, exclusive: boolean): void;
    reserve(request?: ConnectionRequest): Promise<ConnectionReservation>;
}
export declare class ConnectionReservation {
    readonly connection: PostgresClientConnection;
    protected readonly onRelease: (connection: PostgresClientConnection, exclusive: boolean) => void;
    readonly exclusive: boolean;
    released: boolean;
    constructor(connection: PostgresClientConnection, onRelease: (connection: PostgresClientConnection, exclusive: boolean) => void, exclusive: boolean);
    release(): void;
}
interface ScheduledCommand {
    command: Command<unknown>;
    next?: ScheduledCommand;
}
interface PgType {
    oid: number;
    name: string;
    type: string;
}
/**
 * Service to resolve type information for unknown types like enums.
 */
export declare class TypeResolver {
    protected TTL: number;
    protected cache: {
        [oid: number]: {
            type: PgType;
            time: number;
        };
    };
    protected resolveType(connection: PostgresClientConnection, oid: number): Promise<PgType>;
    getType(connection: PostgresClientConnection, oid: number): Promise<PgType>;
}
/**
 * @reflection never
 */
export declare class PostgresClientConnection {
    id: number;
    readonly host: Host;
    protected config: PostgresClientConfig;
    protected onClose: (connection: PostgresClientConnection) => void;
    protected onSent: (bytes: number) => void;
    protected onReceived: (bytes: number) => void;
    protected messageId: number;
    typeResolver: TypeResolver;
    status: ConnectionStatus;
    bufferSize: number;
    cache: {
        [key: string]: Command<unknown>;
    };
    connectingPromise?: Promise<void>;
    commandHead?: ScheduledCommand;
    commandTail?: ScheduledCommand;
    activeCommands: number;
    executedCommands: number;
    /**
     * How many users are currently using this connection, including exclusive users.
     */
    users: number;
    /**
     * How many exclusive users are currently using this connection.
     */
    exclusives: number;
    cleanupTimeout: any;
    protected socket: Socket | TLSSocket;
    transaction?: PostgresDatabaseTransaction;
    responseParser: ResponseParser;
    error?: Error;
    bytesReceived: number;
    bytesSent: number;
    protected boundWrite: (message: Uint8Array) => void;
    constructor(id: number, host: Host, config: PostgresClientConfig, onClose: (connection: PostgresClientConnection) => void, onSent: (bytes: number) => void, onReceived: (bytes: number) => void);
    getCache(key: string): Command<unknown> | undefined;
    setCache(key: string, command: Command<unknown>): void;
    isConnected(): boolean;
    isConnecting(): boolean;
    close(): void;
    run(sql: string): Promise<any>;
    query(sql: string, params: any): any;
    /**
     * When a full message from the server was received.
     */
    protected onResponse(message: Uint8Array): void;
    /**
     * Puts a command on the queue and executes it when queue is empty.
     * A promise is return that is resolved when executed successfully, or rejected
     * when timed out, parser error, or any other error.
     */
    execute<T extends Command<unknown>>(command: T): Promise<ReturnType<T['execute']>>;
    write(message: Uint8Array): void;
    connect(): Promise<void>;
}
export declare class PostgresClient {
    pool: PostgresConnectionPool;
    stats: PostgresStats;
    config: PostgresClientConfig;
    constructor(options: PostgresClientConfig | string);
    reserve(request?: ConnectionRequest): Promise<ConnectionReservation>;
    query(sql: string): Promise<any>;
    execute<T extends Command<unknown>>(command: T): Promise<ReturnType<T['execute']>>;
    close(): void;
}
export {};
export declare type __ΩCommonTypes = any[];
export declare type __ΩResult = any[];
export declare type __ΩConnectionStatus = any[];
export declare type __ΩConnectionRequest = any[];
export declare type __ΩHostType = any[];
